##
##  openpkg -- OpenPKG Tool Chain
##  Copyright (c) 2000-2004 The OpenPKG Project <http://www.openpkg.org/>
##  Copyright (c) 2000-2004 Ralf S. Engelschall <rse@engelschall.com>
##  Copyright (c) 2000-2004 Cable & Wireless <http://www.cw.com/>
##
##  Permission to use, copy, modify, and distribute this software for
##  any purpose with or without fee is hereby granted, provided that
##  the above copyright notice and this permission notice appear in all
##  copies.
##
##  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
##  SUCH DAMAGE.
##

#   decouple from the calling instance
unset OPENPKG_PREFIX

#   FIXME: rse: quick workaround to allow us to use "dev" now: T=$OPENPKG_TOOLS

#   work around broken multiplexer in OpenPKG 2.0.0 and 2.0.1
#   this inhibits the caller from controlling additional and replacement tools through environment variables
if [ ".$OPENPKG_TOOLS_CMDSTACK" = . ]; then
    unset OPENPKG_TOOLS
    unset OPENPKG_TOOLS_APIPATH
    unset OPENPKG_TOOLS_CMDPATH
fi
export OPENPKG_TOOLS
export OPENPKG_TOOLS_APIPATH
export OPENPKG_TOOLS_CMDPATH
export OPENPKG_TOOLS_EXEC
export OPENPKG_TOOLS_APIPATH_EXEC
export OPENPKG_TOOLS_CMDPATH_EXEC
export OPENPKG_TOOLS_TRUN
export OPENPKG_TOOLS_APIPATH_TRUN
export OPENPKG_TOOLS_CMDPATH_TRUN

#   default configuration
OPENPKG_INST="/openpkg-dev /openpkg-20 /openpkg-13 /cw13 /cw12 /cw11 /cw10"
OPENPKG_WORK=${HOME}/work/openpkg
OPENPKG_TEMP=${TMPDIR-/tmp}/${LOGNAME}/openpkg
OPENPKG_MODE=contributor                                        # contributor
OPENPKG_REPO=:pserver:anonymous@cvs.openpkg.org:/e/openpkg/cvs  # contributor
OPENPKG_DIST=ftp://ftp.openpkg.org/contrib/00UPLOAD             # contributor
#OPENPKG_MODE=developer                                         # developer
#OPENPKG_REPO=openpkg-cvs@cvs.openpkg.org:/e/openpkg/cvs        # developer
#OPENPKG_DIST=master.openpkg.org:/e/openpkg/ftp/                # developer
export OPENPKG_INST OPENPKG_WORK OPENPKG_TEMP OPENPKG_MODE OPENPKG_REPO OPENPKG_DIST

#   enforced and user controlled (through package/branch/execute) variables
OPENPKG_SPEC="${OPENPKG_SPEC}"; P="$P" # package, spec file without .spec suffix
OPENPKG_CTAG="${OPENPKG_CTAG}"; B="$B" # cvs tag, branch
OPENPKG_EXEC="${OPENPKG_EXEC}"; E="$E" # exec, instance
OPENPKG_TRUN="${OPENPKG_TRUN}"; T="$T" # trun, tools to run
OPENPKG_SDIR="${OPENPKG_SDIR}"; S="$S" # source dir
export OPENPKG_SPEC P OPENPKG_CTAG B OPENPKG_EXEC E OPENPKG_TRUN T OPENPKG_SDIR S

S="${S:-src}" # chicken-egg hack FIXME

#   use appropriate tools
if [ -x /usr/xpg4/bin/egrep ]; then
    EGREP="/usr/xpg4/bin/egrep"
else
    EGREP="`type -f -p egrep`"
fi
if [ -x /usr/xpg4/bin/awk ]; then
    AWK="/usr/xpg4/bin/awk"
else
    AWK="`type -f -p awk`"
fi

#   a note about CVS HEAD
#   CVS calls the latest version of a file "head". It also supports two
#   hardcoded pseudotags, "BASE" and "HEAD". It is a strange and often
#   unexpected behaviour that checking out the "HEAD" tag is a sticky
#   operation and not identical to checking out the "head" and clear
#   stickyness which requires no -r option to "cvs checkout" or option
#   -A to "cvs up". In contrast to this original behaviour this program
#   uses "HEAD" for input, internal processing and output whenever the
#   user really means "clear stickyness". The "cd" shell script might
#   experience the unlikely event that a directory or file was really
#   updated to the "HEAD" tag. Only in this event the tag is being
#   output as "HEAD_STICKY" which means that this becomes a third
#   "hardcoded" pseudotag.

#   override configuration from ${HOME}/.openpkg-dev.rc (LEGACY)
if [ -f ${HOME}/.openpkg-dev.rc ]; then
    echo "openpkg:dev:WARNING: legacy ~/.openpkg-dev.rc found (please rename to ~/.openpkg/dev.rc)" 1>&2
    . ${HOME}/.openpkg-dev.rc
fi

#   override configuration from ${HOME}/.openpkg/dev.rc (OFFICIAL)
if [ -f ${HOME}/.openpkg/dev.rc ]; then
    . ${HOME}/.openpkg/dev.rc
fi

#   override configuration from command line
verbose=0
quiet=0
version=0
help=0
while [ $# -gt 0 ]; do
    opt=$1
    case ${opt} in
        -*=*) arg=`echo "${opt}" | sed 's/^[-_a-zA-Z0-9]*=//'` ;;
           *) arg='' ;;
    esac
    case ${opt} in
        -v|--verbose ) verbose=1           ;;
        -q|--quiet   ) quiet=1             ;;
        -h|--help    ) help=1              ;;
        -V|--version ) version=1           ;;
        --inst=*     ) OPENPKG_INST=${arg} ;;
        --work=*     ) OPENPKG_WORK=${arg} ;;
        --temp=*     ) OPENPKG_TEMP=${arg} ;;
        --mode=*     ) OPENPKG_MODE=${arg} ;;
        --repo=*     ) OPENPKG_REPO=${arg} ;;
        --dist=*     ) OPENPKG_DIST=${arg} ;;
        --spec=*     ) OPENPKG_SPEC=${arg} ;;
        --ctag=*     ) OPENPKG_CTAG=${arg} ;;
        --exec=*     ) OPENPKG_EXEC=${arg} ;;
        --trun=*     ) OPENPKG_TRUN=${arg} ;;
        --sdir=*     ) OPENPKG_SDIR=${arg} ;;
        -*           ) help="Invalid option \`${opt}'"; break ;;
        *            ) break              ;;
    esac
    shift
done

#   ------------------------------------------------------------

#   dump a function (optionally only the inner code part)
dumpfunc () {
    if [ ".$1" = ".-i" ]; then
        shift
        type $1 \
        | ${AWK} -v cmd=$1 'BEGIN { f=0 }
               /^ / { if (f==1) { print $0 } }
               /^[a-zA-Z0-9_]+ is a function/ { f=1 }'
    else
        type $1 \
        | ${AWK} -v cmd=$1 'BEGIN { f=0 }
               // { if (f==1) { print $0 } }
               /^[a-zA-Z0-9_]+ is a function/ { print "unalias "cmd" >/dev/null 2>&1"; f=1 }'
    fi
}

#   print a warning
warn () {
    echo "openpkg:dev:WARNING: $@" 1>&2
}

#   print an error
error () {
    echo "openpkg:dev:ERROR: $@" 1>&2
}

#   print an error and die
die () {
    error "$@"
    exit 1
}

makedir () {
    perm="$1"
    dir="$2"
    set -- `echo ":${dir}" | sed -e 's/^:\//%/' -e 's/^://' -e 's/\// /g' -e 's/^%/\//'`
    pathcomp=''
    for d in ${1+"$@"}; do
        pathcomp="${pathcomp}${d}"
        if [ ! -d "${pathcomp}" ]; then
            mkdir ${pathcomp} || errstatus=$?
            chmod ${perm} ${pathcomp} || errstatus=$?
        fi
        pathcomp="${pathcomp}/"
    done
}

#   provide a unified which(1) command, because vendor implementations differ in error results.
uwhich () {
    _rv=""
    for _tool in $*; do
        _ifs="$IFS"; IFS=":"
        for _path in $PATH; do
            if [ -f "$_path/$_tool" ]; then
                _rv="$_path/$_tool"
                break
            fi
        done
        IFS="$_ifs"; unset _ifs
        if [ ".$_rv" != . ]; then
            break
        fi
    done
    echo $_rv
    unset _rv
}

#   init MPX, RPM, OPENPKG_TOOLS_(API|CMD)PATH and RPMCMD
#   to the correct values for a given instance
#   see http://cvs.openpkg.org/chngview?cn=14911
initmpx ()
{
    if [ -x $1/bin/openpkg -a -x $1/libexec/openpkg/rpm ]; then
        #OpenPKG v2.x use "openpkg" multiplexer
        MPX="$1/bin/openpkg"
        RPM="rpm"
    elif [ -x $1/bin/rpm -a ! -x $1/libexec/openpkg/rpm ]; then
        #OpenPKG v1.x direct command execution
        MPX=""
        RPM="$1/bin/rpm"
    else
        # OpenPKG N/A
        MPX=""
        RPM=""
    fi
    RPMCMD="$MPX${MPX:+ }$RPM"
}

openpkgi () {
    if [ ".$1" = ".-e" ]; then
        if [ ".$E" != . ]; then
            initmpx $E
        else
            error "$0:ERROR: don't know which rpm to exec (E unset)" 1>&2
            return 1
        fi
        initmpx $E
        if [ ".$OPENPKG_TOOLS_EXEC" = . ];         then unset OPENPKG_TOOLS;         else OPENPKG_TOOLS="${OPENPKG_TOOLS_EXEC}";                 fi
        if [ ".$OPENPKG_TOOLS_APIPATH_EXEC" = . ]; then unset OPENPKG_TOOLS_APIPATH; else OPENPKG_TOOLS_APIPATH="${OPENPKG_TOOLS_APIPATH_EXEC}"; fi
        if [ ".$OPENPKG_TOOLS_CMDPATH_EXEC" = . ]; then unset OPENPKG_TOOLS_CMDPATH; else OPENPKG_TOOLS_CMDPATH="${OPENPKG_TOOLS_CMDPATH_EXEC}"; fi
    elif [ ".$1" = ".-t" ]; then
        if [ ".$T" != . ]; then
            initmpx $T
        else
            echo "$0:ERROR: don't know which tools to run (T unset)" 1>&2
            return 1
        fi
        initmpx $T
        if [ ".$OPENPKG_TOOLS_TRUN" = . ];         then unset OPENPKG_TOOLS;         else export OPENPKG_TOOLS="${OPENPKG_TOOLS_TRUN}";                 fi
        if [ ".$OPENPKG_TOOLS_APIPATH_TRUN" = . ]; then unset OPENPKG_TOOLS_APIPATH; else export OPENPKG_TOOLS_APIPATH="${OPENPKG_TOOLS_APIPATH_TRUN}"; fi
        if [ ".$OPENPKG_TOOLS_CMDPATH_TRUN" = . ]; then unset OPENPKG_TOOLS_CMDPATH; else export OPENPKG_TOOLS_CMDPATH="${OPENPKG_TOOLS_CMDPATH_TRUN}"; fi
    elif [ ".$1" = ".-i" ]; then
        shift
        initmpx $1
    else
        error "$0:ERROR: unknown option $1; USAGE: openpkgi [-e|-t|-i inst]" 1>&2
        return 1
    fi
    if [ ".${OPENPKG_TOOLS_CMDPROG}" = . ]; then
        error "OPENPKG_TOOLS_CMDPROG not set"
        return 1
    fi
    if [ ".${OPENPKG_TOOLS_CMDNAME}" = . ]; then
        error "OPENPKG_TOOLS_CMDNAME not set"
        return 1
    fi
    shift
    if [ ".$1" = .dev ]; then
        shift
        case $1 in
            package | pa* ) shift; openpkg_dev_package "$@"; cd . ;;
            branch  | br* ) shift; openpkg_dev_branch  "$@"; cd . ;;
            execute | ex* ) shift; openpkg_dev_execute "$@"; cd . ;;
            tools   | to* ) shift; openpkg_dev_tools   "$@"; cd . ;;
            srcdir  | sr* ) shift; openpkg_dev_srcdir  "$@"; cd . ;;
                        * ) command $T/bin/openpkg dev "$@"
        esac
    elif [ ".$1" = .rpm ]; then
        shift
        if [ ".${RPMCMD}" = . ]; then
            error "$0:ERROR: no openpkg rpm found at instance \"$E\""
            return 1
        fi
        eval "HOME=\${OPENPKG_WORK} command ${RPMCMD} \"\$@\""
    else
        if [ ".${MPX}" = . ]; then
            error "$0:ERROR: no openpkg multiplexer found at instance \"$E\""
            return 1
        fi
        eval "HOME=\${OPENPKG_WORK} command ${MPX} \"\$@\""
    fi
}

openpkgE () {
    openpkgi -e "$@"
}

openpkgT () {
    openpkgi -t "$@"
}

openpkg () {
    if [ ".$1" = .dev ]; then
        openpkgi -t "$@"
    elif [ ".$1" = .rpm ]; then
        openpkgi -e "$@"
    else
        echo "$0:ERROR: don't know which openpkg multiplexer to call (unknown command $1)" 1>&2
        return 1
    fi
}

rpmi () {
    local mode
    mode=$1
    shift
    openpkgi $mode rpm "$@"
}

rpmE () {
    rpmi -e "$@"
}

rpmT () {
    rpmi -t "$@"
}

rpm () {
    openpkg rpm "$@"
}

#   smart dealing with temporary root privileges
root () {
    if [ ".$1" = . ]; then
        ssh -t -x root@${HOSTNAME} builtin cd ${PWD} \&\& ${SHELL}
    elif [ ".$1" = .rpm ]; then
        initmpx $E
        shift
        if [ ".${RPMCMD}" = . ]; then
            echo "$0:ERROR: no openpkg rpm found at instance \"$E\""
            return 1
        fi
        ssh -t -x root@${HOSTNAME} builtin cd ${PWD} \&\& HOME=${OPENPKG_WORK} command ${RPMCMD} "$@"
    elif [ ".$1" = .openpkg ]; then
        initmpx $E
        shift
        if [ ".${MPX}" = . ]; then
            echo "$0:ERROR: no openpkg multiplexer found at instance \"$E\""
            return 1
        fi
        ssh -t -x root@${HOSTNAME} builtin cd ${PWD} \&\& HOME=${OPENPKG_WORK} command ${MPX} "$@"
    else
        ssh -x root@${HOSTNAME} builtin cd ${PWD} \&\& "$@"
    fi
}

cvs () {
    cvs_colorize="cat"
    case "$1" in
        di|dif|diff   ) [ ".`uwhich cvs-diff-colorize`"   != . ] && cvs_colorize="cvs-diff-colorize"   ;;
        up|upd|update ) [ ".`uwhich cvs-update-colorize`" != . ] && cvs_colorize="cvs-update-colorize" ;;
    esac
    HOME=${OPENPKG_WORK} command cvs ${1+"$@"} | $cvs_colorize
}

setup () {
    #   honor optional parameters [[[spec]ctag]exec]; use "" to specify a empty parameter and use it's default
    [ ".$1" != . ] && OPENPKG_SPEC="$1"
    [ ".$2" != . ] && OPENPKG_CTAG="$2"
    [ ".$3" != . ] && OPENPKG_EXEC="$3"

    #   OPENPKG_SPEC in setup() is only a flag to indicate minimal checkout

    #   canonify OPENPKG_CTAG, default to HEAD
    canonifybranch "${OPENPKG_CTAG}"
    OPENPKG_CTAG="${RV:-HEAD}"

    #   canonify OPENPKG_EXEC, default to HEAD
    canonifybranch "${OPENPKG_EXEC:-$OPENPKG_CTAG}"
    matchinstance "${RV-HEAD}"
    OPENPKG_EXEC="${RV}"

    #   set P,B,E without having "cd()" already in place
    P="${OPENPKG_SPEC}"
    B="${OPENPKG_CTAG}"
    E="${OPENPKG_EXEC}"
    T="${OPENPKG_TRUN}"
    S="${OPENPKG_SDIR}"

    #   sanity check
    case "${OPENPKG_WORK}" in
        /?* ) ;;
          * ) die "working area ${OPENPKG_WORK} seems to be strange" ;;
    esac
    if [ -d "${OPENPKG_WORK}" ]; then
        die "working area ${OPENPKG_WORK} already exists"
    fi

    #   create a working area
    echo "++ creating working area ${OPENPKG_WORK}"
    if [ ! -d "${OPENPKG_WORK}" ]; then
        makedir 755 "${OPENPKG_WORK}" || die "failed to create ${OPENPKG_WORK} directory"
    fi

    #   sane environment
    umask 022

    #   create a .cvspass
    echo "++ creating CVS password file (${OPENPKG_WORK}/.cvspass)"
    sed -e 's;^ *;;' <<EOF >"${OPENPKG_WORK}/.cvspass"
        /1 ${OPENPKG_REPO} A
EOF

    #   create a .cvsrc
    echo "++ creating CVS configuration (${OPENPKG_WORK}/.cvsrc)"
    updatedir=""; [ ".${P}" = . ] && updatedir="-d"
    sed -e 's;^ *;;' <<EOF >"${OPENPKG_WORK}/.cvsrc"
        cvs -z4
        checkout -P
        update -P $updatedir
        diff -u -d
        rdiff -u
EOF

    echo "++ creating CVS file ignore list (${OPENPKG_WORK}/.cvsignore)"
    sed -e 's;^ *;;' <<EOF >"${OPENPKG_WORK}/.cvsignore"
        .[a-za-Z0-9]*
        00README
        dst
        pkg
EOF

    #   create a .rpmmacros, save existing, overwrite only if different
    echo "++ creating RPM configuration (${OPENPKG_WORK}/.rpmmacros)"
    sed -e 's;^ *;;' <<EOF >"${OPENPKG_WORK}/.rpmmacros"
        #   source areas
        %_sourcedir    %(echo \${OPENPKG_WORK})/dst/%{name}
        %_specdir      %(echo \${OPENPKG_WORK})/%(echo \$S)/%{name}

        #   temporary areas
        %_builddir     %(echo \${OPENPKG_TEMP})
        %_tmppath      %(echo \${OPENPKG_TEMP})

        #   target areas
        %_rpmdir       %(echo \${OPENPKG_WORK})/pkg/bin
        %_srcrpmdir    %(echo \${OPENPKG_WORK})/pkg/src
EOF

    echo "++ creating directory structure (${OPENPKG_REPO}/{dst,pkg})"
    #   special overridden setup for dev.de.cw.net domain
    case `hostname` in
        *.dev.de.cw.net )
            ( echo ""
              echo "#   ==== Overide settings for C&W IS Development Team ==="
              echo ""
              echo "#   use local mirror first"
              echo "%l_fetch_mirror_0 ftp://ftp.openpkg.org/sources/DST/%{name}/"
              echo ""
              echo "#   use shared storage"
              echo "%_sourcedir       /e/openpkg/DST/%{name}"
              echo "%_rpmdir          /e/openpkg/PKG/bin"
              echo "%_srcrpmdir       /e/openpkg/PKG/src"
            ) >>${OPENPKG_WORK}/.rpmmacros
            if [ ! -d "${OPENPKG_WORK}/dst" ]; then
                ln -s /e/openpkg/DST "${OPENPKG_WORK}/dst" || die "failed to link to ${OPENPKG_WORK}/dst directory"
            fi
            if [ ! -d "${OPENPKG_WORK}/pkg" ]; then
                ln -s /e/openpkg/PKG "${OPENPKG_WORK}/pkg" || die "failed to link to ${OPENPKG_WORK}/pkg directory"
            fi
            ;;
        * )
            if [ ! -d "${OPENPKG_WORK}/dst" ]; then
                makedir 755 "${OPENPKG_WORK}/dst" || die "failed to create ${OPENPKG_WORK}/dst directory"
            fi
            if [ ! -d "${OPENPKG_WORK}/pkg/src" ]; then
                makedir 755 ${OPENPKG_WORK}/pkg/src || die "failed to create ${OPENPKG_WORK}/pkg/src directory"
            fi
            if [ ! -d "${OPENPKG_WORK}/pkg/bin" ]; then
                makedir 755 ${OPENPKG_WORK}/pkg/bin || die "failed to create ${OPENPKG_WORK}/pkg/bin directory"
            fi
    esac

    #   checkout the CVS module(s)
    if [ ".${P}" = . ]; then
        echo "++ checking out source areas (${OPENPKG_REPO}/{adm,doc,re,src,web})"
        ( builtin cd "${OPENPKG_WORK}"
          HOME="${OPENPKG_WORK}"
          export HOME
          cvs -q -d "${OPENPKG_REPO}" checkout openpkg-adm openpkg-doc openpkg-re openpkg-web openpkg-tools
          if [ ".${B:-HEAD}" = .HEAD ]; then
              cvs -q -d "${OPENPKG_REPO}" checkout openpkg-src
          else
              cvs -q -d "${OPENPKG_REPO}" checkout -r ${B} openpkg-src
          fi
        )
    else
        echo "++ checking out minimal source areas (${OPENPKG_REPO}/{re,src})"
        ( builtin cd "${OPENPKG_WORK}"
          HOME="${OPENPKG_WORK}"
          export HOME
          cvs -q -d "${OPENPKG_REPO}" checkout openpkg-re
          if [ ".${B:-HEAD}" = .HEAD ]; then
              cvs -q -d "${OPENPKG_REPO}" checkout openpkg-src/00README
              cvs -q -d "${OPENPKG_REPO}" checkout openpkg-src/${P}
          else
              cvs -q -d "${OPENPKG_REPO}" checkout -r ${B} openpkg-src/00README
              cvs -q -d "${OPENPKG_REPO}" checkout -r ${B} openpkg-src/${P}
          fi
        )
    fi

    echo "++ creating README file (${OPENPKG_REPO}/00README)"
    help

    #   cvs with RSE patches reads ~/.cvsrc, the .cvsrc file in the
    #   current directory and all directories towards / and merges
    #   all the options it finds.
    #   FIXME
    #   - patch the RSE patch to disable the upward search -> bad for existing binaries
    #   - use -f option to ignore .cvsrc -> requires this script to provide the options directly
    #   - temporary solution is to just print a warning
    cvs --version | grep RSE >/dev/null && if [ -f $HOME/.cvsrc ]; then
        warn "you use cvs with RSE patches, your ~/.cvsrc may collide with ${OPENPKG_WORK}/.cvsrc"
    fi
}

update () {
    echo "++ updating ${OPENPKG_REPO}/{src,web,doc,re,adm}"
    ( builtin cd ${OPENPKG_WORK} || die "cannot cd to ${OPENPKG_WORK}"
      HOME=${OPENPKG_WORK}
      export HOME
      cvs -d ${OPENPKG_REPO} update
    )
}

package () {
    warn "running \"package\" is useless until called from within interactive development shell"
}

openpkg_dev_package () {
    local package=""
    if [ ".$1" = .-q -o ".$1" = .--query ]; then
        package=`echo ${PWD} | sed -e "s;^${OPENPKG_WORK}/$S/;;" -e 's;/.*$;;'`
        echo "${package}"
    else
        package="$1"
        if [ ".${package}" = . ]; then
            OPENPKG_SPEC=""
            cd "${OPENPKG_WORK}/$S"
        else
            if [ ! -d "${OPENPKG_WORK}/$S" ]; then
                error "${OPENPKG_WORK}/$S directory not found"
                return 1
            fi

            ( builtin cd ${OPENPKG_WORK}/$S && HOME=${OPENPKG_WORK} cvs up -d ${package} >/dev/null 2>&1 )
            if [ ! -d "${OPENPKG_WORK}/$S/${package}" ]; then
                error "${OPENPKG_WORK}/$S/${package} directory not found"
                return 1
            fi

            OPENPKG_SPEC="${package}"
            cd "${OPENPKG_WORK}/$S/${package}"
        fi
    fi
}

branch () {
    warn "running \"branch\" is useless until called from within interactive development shell"
}

openpkg_dev_branch () {
    local branch=""
    if [ ".$1" = .-q -o ".$1" = .--query ]; then
        cvstagfiles=""
        if [ -r CVS/Entries ]; then
            cvstagfiles="${cvstagfiles} CVS/Entries"
        fi
        if [ -r CVS/Tag ]; then
            cvstagfiles="${cvstagfiles} CVS/Tag"
        fi
        if [ ".${cvstagfiles}" != . ]; then
            branch=`cat ${cvstagfiles} \
            | sed -e 's;//THEAD$;//THEAD_STICKY;' -e 's;//$;//THEAD;' -n -e 's;^.*//T;;p' \
            | sort | uniq | tr '\n' ':' | sed -e 's;:$;;'`
        fi
        echo "${branch}"
    else
        branch="$1"
        if [ ".${branch}" = . ]; then
            return
        fi
        #FIXME canonifybranch "${1}"
        if [ ".${branch}" = .HEAD ]; then
            HOME=${OPENPKG_WORK} cvs update -A
        else
            HOME=${OPENPKG_WORK} cvs update -r ${branch}
        fi

        OPENPKG_CTAG=${branch}
        cd .
    fi
}

execute () {
    warn "running \"execute\" is useless until called from within interactive development shell"
}

openpkg_dev_execute () {
    local execute=""
    if [ ".$1" = .-q -o ".$1" = .--query ]; then
        #FIXME matchinstance "${B}"
        echo $E
    else
        execute="$1"
        if [ ".${execute}" = . ]; then
            return
        fi
        initmpx ${execute}
        if [ ".${RPMCMD}" = . ]; then
            error "${execute} is not a OpenPKG instance"
            return 1
        fi

        OPENPKG_EXEC="${execute}"
        cd .
    fi
}

tools () {
    warn "running \"tools\" is useless until called from within interactive development shell"
}

openpkg_dev_tools () {
    if [ ".$1" = .-q -o ".$1" = .--query ]; then
        echo $T
    else
        OPENPKG_TRUN="$1"
        T="${OPENPKG_TRUN}"
    fi
}

srcdir () {
    warn "running \"srcdir\" is useless until called from within interactive development shell"
}

openpkg_dev_srcdir () {
    if [ ".$1" = .-q -o ".$1" = .--query ]; then
        S=`pwd | sed -e 's;^;X;' -e 's;^X.*/\(src\.[^/][^/]*\).*$;\1;' -e 's;^X.*;src;'`
        echo "$S"
    else
        if [ -d "${OPENPKG_WORK}/src.$1" ]; then
            OPENPKG_SDIR="src.$1"
            cd .
        else
            OPENPKG_SDIR="src"
            cd .
        fi
    fi
}

#   convert any (branch|instance|name) into branch and return ${RV}
#
#   Branch              Instance   Name
#   ------------------- ---------- --------
#   HEAD                /cw        CURRENT
#   OPENPKG_1_STABLE    /cw1       1-STABLE
#   OPENPKG_1_2_SOLID   /cw12      1.2
#   OPENPKG_1_1_SOLID   /cw11      1.1
#   OPENPKG_1_0_SOLID   /cw10      1.0
#
canonifybranch () {
    #   remove surrounding whitespaces, take first arg only, toupper
    RV=`echo ${1} | cut -f 1 -d" " | tr '[a-z]' '[A-Z]'`

    #   empty in, empty return (mapping emtpy to HEAD is the task of the caller)
    [ ".${RV}" = . ] && return

    #   strip off any abbreviated form of a leading OPENPKG_
    RV=`echo "${RV}" | sed -e 's;^O\{0,1\}P\{0,1\}E\{0,1\}N\{0,1\}P\{0,1\}K\{0,1\}G\{0,1\}_\{0,1\};;'`

    #   transform instance into branch
    RV=`echo "${RV}" | sed -e 's;^/[A-Z]\{1,\}$;HEAD;' -e 's;^/[A-Z]\{1,\}\([1-9]\);\1_;'`

    #   replace dots from (SOLID name) and dashes (STABLE name) with underscore for convenience
    RV=`echo "${RV}" | sed -e 's;[\.-];_;g'`

    #   invalid branch, empty return (mapping emtpy to HEAD is the task of the caller)
    [ ".${RV}" = . ] && return

    #   check for HEAD (branch) aka CURRENT (name)
    echo "CURRENT" | ${EGREP} "^${RV}" >/dev/null 2>&1 && RV="HEAD"
    echo "HEAD"    | ${EGREP} "^${RV}" >/dev/null 2>&1 && RV="HEAD"

    if [ .${RV} != ".HEAD" ]; then

        #   extract branchpoint (1, 1_1 ...) and optional direction
        BRANBP=`echo "${RV}" | sed -e 's;^\([1-9][0-9]*\(_\{0,1\}[0-9]*\)\)\(.*\)$;\1;' -e 's;_*$;;'`
        BRANDR=`echo "${RV}" | sed -e 's;^\([1-9][0-9]*\(_\{0,1\}[0-9]*\)\)\(.*\)$;\3;' -e 's;^_*;;'`

        #   select direction, check optional direction
        echo ${BRANBP} | grep '_' >/dev/null
        if [ $? -eq 0 ]; then
            RV="OPENPKG_${BRANBP}_SOLID"
        else
            RV="OPENPKG_${BRANBP}_STABLE"
        fi

        unset BRANBP
        unset BRANDR
    fi
}

#   find matching instance with a executeable rpm for a given branch $1; return "" if none found
#   $1 = requested branch
#   $2 = optional mode one of "exact", "best", "any"; if omitted, defaults to "any"
#        exact = find executable rpm exactly matching branch
#        any   = take first executable rpm
matchinstance () {
    mode="${2:-any}"
    [ ".$mode" != .exact -a ".$mode" != .any ] && die "matchinstance() assertion"

    INSTANCE="$1"
    if [ ".$mode" = .exact ]; then
        echo "${INSTANCE}" | ${EGREP} "^/" >/dev/null #PATH
        if [ $? -eq 0 ]; then
            initmpx ${INSTANCE}
            if [ ".${RPMCMD}" != . ]; then
                RV="${INSTANCE}"
                unset INSTANCE
                return
            else
                RV=""
                unset INSTANCE
                return
            fi
        fi
    fi

    INSTANCE=`echo "$1" | sed -e 's;^OPENPKG_;;' -e 's;_STABLE$;;' -e 's;_SOLID$;;' -e 's;^HEAD$;;' -e 's;_;.;g'`
    RE=""
    echo "${INSTANCE}" | ${EGREP} -i "^head$"                    >/dev/null && RE="^[0-9]{8}$"     #HEAD (any)
    echo "${INSTANCE}" | ${EGREP} "^[0-9]{8}$"                   >/dev/null && RE="^${INSTANCE}$"  #HEAD
    echo "${INSTANCE}" | ${EGREP} "^[0-9]{1,7}$"                 >/dev/null && RE="^${INSTANCE}\." #STABLE
    echo "${INSTANCE}" | ${EGREP} "^[0-9]+\.[0-9]{8}$"           >/dev/null && RE="^${INSTANCE}$"  #SNAPSHOT
    echo "${INSTANCE}" | ${EGREP} "^[0-9]+\.[0-9]{1,7}$"         >/dev/null && RE="^${INSTANCE}"   #SOLID (any UPD)
    echo "${INSTANCE}" | ${EGREP} "^[0-9]+\.[0-9]{1,7}\.[0-9]+$" >/dev/null && RE="^${INSTANCE}$"  #SOLID
    RE="`echo ${RE} | sed -e 's;\.;\\.;g'`"

    for INSTANCE in ${OPENPKG_INST}; do
        initmpx ${INSTANCE}
        if [ ".${RPMCMD}" != . ]; then
            VERSION=`${RPMCMD} -q --qf "%{VERSION}" openpkg`
            echo ${VERSION} | ${EGREP} "$RE" >/dev/null
            if [ $? -eq 0 ]; then
                RV="${INSTANCE}"
                unset INSTANCE
                return
            fi
        fi
    done
    if [ ".$mode" = .any ]; then
        OPENPKG_INST="${OPENPKG_INST} `uwhich openpkg | sed -e 's;/bin/openpkg$;;'`"
        OPENPKG_INST="${OPENPKG_INST} `uwhich rpm | sed -e 's;/bin/rpm$;;'`"
        for INSTANCE in ${OPENPKG_INST}; do
            initmpx ${INSTANCE}
            if [ ".${RPMCMD}" != . ]; then
                RV="${INSTANCE}"
                unset INSTANCE
                return
            fi
        done
    fi
    RV=""
    unset INSTANCE
}

bashrcprolog () {
    #   use original .bashrc as a baseline
    [ -f ${HOME}/.bashrc ] && source ${HOME}/.bashrc

    #   use SSH for CVS
    CVS_RSH=ssh
    export CVS_RSH

    #   import OpenPKG environment
    #eval `$E/etc/rc --eval openpkg env` #FIXME no longer needed (i.e. MANPATH); anyway: wrong place here!

    #   prompt to indicate openpkg dev bash environment
    declare -a DIRSTACKREV=()
    cd () {
        local CD_FUNC_PX CD_FUNC_PD CD_FUNC_BX CD_FUNC_BD CD_FUNC_EX CD_FUNC_ED CD_FUNC_TX CD_FUNC_TD CD_FUNC_SX CD_FUNC_SD CD_FUNC_IX CD_FUNC_ID
        if [ ".$1" = .- ]; then
            #   go to previous working directory on forward directory stack
            #   and move this directory onto the reverse directory stack
            if [ ${#DIRSTACK[*]} -gt 1 ]; then
                workdir="${DIRSTACK[0]}"
                DIRSTACKREV[${#DIRSTACKREV[*]}]="${workdir}"
                popd >/dev/null
            else
                echo "cd: no more previous working directories on forward directory stack" 1>&2
                return 1
            fi
        elif [ ".$1" = .+ ]; then
            #   go to previous working directory on reverse directory stack
            #   and move this directory onto the forward directory stack
            if [ ${#DIRSTACKREV[*]} -gt 0 ]; then
                local i=$((${#DIRSTACKREV[*]} - 1))
                eval "pushd ${DIRSTACKREV[$i]} >/dev/null"
                unset DIRSTACKREV[$i]
            else
                echo "cd: no more previous working directories on reverse directory stack" 1>&2
                return 1
            fi
        else
            #   go to next working directory
            #   and move this directory onto the forward directory stack
            if [ ".$1" = . ]; then
                if [ ".${OPENPKG_SDIR}" = . ]; then
                    workdir="${OPENPKG_WORK}/src/${OPENPKG_SPEC:+/}${OPENPKG_SPEC}"
                else
                    workdir="${OPENPKG_WORK}/${OPENPKG_SDIR}/${OPENPKG_SPEC:+/}${OPENPKG_SPEC}"
                fi
            else
                workdir="$1"
            fi
            if [ -d "${workdir}" ]; then
                pushd ${workdir} >/dev/null
                if [ ${#DIRSTACK[*]} -ge 2 -a ${DIRSTACK[0]} = ${DIRSTACK[1]} ]; then
                    popd >/dev/null
                fi
            fi
            DIRSTACKREV=()
        fi

        P=`openpkg_dev_package --query`
        # P holds detection
        if [ ".${OPENPKG_SPEC}" = . ]; then
            if [ ".${P}" = . ]; then
                # nothing forced, nothing detected; keep nothing; print nothing
                :
            else
                # nothing forced, something detected; keep detected; print detected normal
                CD_FUNC_PX="P=${P}"
                CD_FUNC_PD="P=${P}"
            fi
        else
            if [ ".${P}" = . ]; then
                # forced, nothing detected; take forced; print forced black (aka trailing dot)
                P="${OPENPKG_SPEC}"
                CD_FUNC_PX="[1mP=${P}[m"
                CD_FUNC_PD="P=${P}."
            else
                if [ ".${OPENPKG_SPEC}" = ".${P}" ]; then
                    # forced; detected and matched; keep; print black (aka trailing dot)
                    CD_FUNC_PX="[1mP=${P}[m"
                    CD_FUNC_PD="P=${P}."
                else
                    # forced; detected and mismatch; keep forced; print forced red (aka trailing exclamation mark)
                    CD_FUNC_PX="[31mP=${OPENPKG_SPEC}[m"
                    CD_FUNC_PD="P=${OPENPKG_SPEC}!"
                    P="${OPENPKG_SPEC}"
                fi
            fi
        fi
        # P has ultimate content

        B=`openpkg_dev_branch --query`
        # B holds detection
        if [ ".${OPENPKG_CTAG}" = . ]; then
            if [ ".$B" = . ]; then
                # nothing forced, nothing detected; keep nothing; print nothing
                :
            else
                # nothing forced, something detected; keep detected; print detected normal
                CD_FUNC_BX="B=$B"
                CD_FUNC_BD="B=$B"
            fi
        else
            if [ ".$B" = . ]; then
                # forced, nothing detected; take forced; print forced black (aka trailing dot)
                B="${OPENPKG_CTAG}"
                CD_FUNC_BX="[1mB=$B[m"
                CD_FUNC_BD="B=$B."
            else
                if [ ".${OPENPKG_CTAG}" = ".$B" ]; then
                    # forced; detected and matched; keep; print black (aka trailing dot)
                    CD_FUNC_BX="[1mB=$B[m"
                    CD_FUNC_BD="B=$B."
                else
                    # forced; detected and mismatch; keep forced; print forced red (aka trailing exclamation mark)
                    CD_FUNC_BX="[31mB=${OPENPKG_CTAG}[m"
                    CD_FUNC_BD="B=${OPENPKG_CTAG}!"
                    B="${OPENPKG_CTAG}"
                fi
            fi
        fi
        # B has ultimate content

        E=`openpkg_dev_execute --query`
        # E holds detection
        if [ ".${OPENPKG_EXEC}" = . ]; then
            if [ ".$E" = . ]; then
                # nothing forced, nothing detected; keep nothing; print nothing
                :
            else
                # nothing forced, something detected; keep detected; print detected normal
                CD_FUNC_EX="E=$E"
                CD_FUNC_ED="E=$E"
            fi
        else
            if [ ".$E" = . ]; then
                # forced, nothing detected; take forced; print forced black (aka trailing dot)
                E="${OPENPKG_EXEC}"
                CD_FUNC_EX="[1mE=$E[m"
                CD_FUNC_ED="E=$E."
            else
                if [ ".${OPENPKG_EXEC}" = ".$E" ]; then
                    # forced; detected and matched; keep; print black (aka trailing dot)
                    CD_FUNC_EX="[1mE=$E[m"
                    CD_FUNC_ED="E=$E."
                else
                    # forced; detected and mismatch; keep forced; print forced red (aka trailing exclamation mark)
                    CD_FUNC_EX="[1mE=${OPENPKG_EXEC}[m"
                    CD_FUNC_ED="E=${OPENPKG_EXEC}!"
                    E="${OPENPKG_EXEC}"
                fi
            fi
        fi
        # E has ultimate content

        T=`openpkg_dev_execute --query`
        # T holds detection
        if [ ".${OPENPKG_TRUN}" = . ]; then
            if [ ".$T" = . ]; then
                # nothing forced, nothing detected; keep nothing; print nothing
                :
            else
                # nothing forced, something detected; keep detected; print detected normal
                CD_FUNC_TX="T=$T"
                CD_FUNC_TD="T=$T"
            fi
        else
            if [ ".$T" = . ]; then
                # forced, nothing detected; take forced; print forced black (aka trailing dot)
                T="${OPENPKG_TRUN}"
                CD_FUNC_TX="[1mT=$T[m"
                CD_FUNC_TD="T=$T."
            else
                if [ ".${OPENPKG_TRUN}" = ".$T" ]; then
                    # forced; detected and matched; keep; print black (aka trailing dot)
                    CD_FUNC_TX="[1mT=$T[m"
                    CD_FUNC_TD="T=$T."
                else
                    # forced; detected and mismatch; keep forced; print forced red (aka trailing exclamation mark)
                    CD_FUNC_TX="[1mT=${OPENPKG_TRUN}[m"
                    CD_FUNC_TD="T=${OPENPKG_TRUN}!"
                    T="${OPENPKG_TRUN}"
                fi
            fi
        fi
        # T has ultimate content

        S=`openpkg_dev_srcdir --query`
        # S holds detection
        if [ ".${OPENPKG_SDIR}" = . ]; then
            if [ ".$S" = . ]; then
                # nothing forced, nothing detected; keep nothing; print nothing
                :
            else
                # nothing forced, something detected; keep detected; print detected normal
                CD_FUNC_SX="S=$S"
                CD_FUNC_SD="S=$S"
            fi
        else
            if [ ".$S" = . ]; then
                # forced, nothing detected; take forced; print forced black (aka trailing dot)
                S="${OPENPKG_SDIR}"
                CD_FUNC_SX="[1mS=$S[m"
                CD_FUNC_SD="S=$S."
            else
                if [ ".${OPENPKG_SDIR}" = ".$S" ]; then
                    # forced; detected and matched; keep; print black (aka trailing dot)
                    CD_FUNC_SX="[1mS=$S[m"
                    CD_FUNC_SD="S=$S."
                else
                    # forced; detected and mismatch; keep forced; print forced red (aka trailing exclamation mark)
                    CD_FUNC_SX="[1mS=${OPENPKG_SDIR}[m"
                    CD_FUNC_SD="S=${OPENPKG_SDIR}!"
                    S="${OPENPKG_SDIR}"
                fi
            fi
        fi
        # S has ultimate content

        CD_FUNC_IX=""
        if [ ".$CD_FUNC_PX" != . ]; then
            CD_FUNC_IX="${CD_FUNC_IX}${CD_FUNC_IX:+ }${CD_FUNC_PX}"
        fi
        if [ ".$CD_FUNC_BX" != . ]; then
            CD_FUNC_IX="${CD_FUNC_IX}${CD_FUNC_IX:+ }${CD_FUNC_BX}"
        fi
        if [ ".$CD_FUNC_EX" != . ]; then
            CD_FUNC_IX="${CD_FUNC_IX}${CD_FUNC_IX:+ }${CD_FUNC_EX}"
        fi
        if [ ".$CD_FUNC_TX" != . ]; then
            CD_FUNC_IX="${CD_FUNC_IX}${CD_FUNC_IX:+ }${CD_FUNC_TX}"
        fi
        if [ ".$CD_FUNC_SX" != . ]; then
            CD_FUNC_IX="${CD_FUNC_IX}${CD_FUNC_IX:+ }${CD_FUNC_SX}"
        fi
        if [ ".$CD_FUNC_IX" != . ]; then
            CD_FUNC_IX=" [${CD_FUNC_IX}]"
        fi

        CD_FUNC_ID=""
        if [ ".$CD_FUNC_PD" != . ]; then
            CD_FUNC_ID="${CD_FUNC_ID}${CD_FUNC_ID:+ }${CD_FUNC_PD}"
        fi
        if [ ".$CD_FUNC_BD" != . ]; then
            CD_FUNC_ID="${CD_FUNC_ID}${CD_FUNC_ID:+ }${CD_FUNC_BD}"
        fi
        if [ ".$CD_FUNC_ED" != . ]; then
            CD_FUNC_ID="${CD_FUNC_ID}${CD_FUNC_ID:+ }${CD_FUNC_ED}"
        fi
        if [ ".$CD_FUNC_TD" != . ]; then
            CD_FUNC_ID="${CD_FUNC_ID}${CD_FUNC_ID:+ }${CD_FUNC_TD}"
        fi
        if [ ".$CD_FUNC_SD" != . ]; then
            CD_FUNC_ID="${CD_FUNC_ID}${CD_FUNC_ID:+ }${CD_FUNC_SD}"
        fi
        if [ ".$CD_FUNC_ID" != . ]; then
            CD_FUNC_ID=" [${CD_FUNC_ID}]"
        fi

        if [ ".${TERM}" = .xterm ]; then
            PS1="${USER}@${HOSTNAME}:${PWD}${CD_FUNC_IX}\n${OPENPKG_MODE}\$ "
            echo -n "]0;openpkg dev ${USER}@${HOSTNAME}:${PWD}${CD_FUNC_ID}"
        else
            PS1="${USER}@${HOSTNAME}:${PWD}${CD_FUNC_ID}\n${OPENPKG_MODE}\$ "
        fi
    }

    alias opd="openpkg dev"
}

bashrcepilog () {
    #   sane environment
    umask 022

    #   prepare for a individual package
    if [ ".$1" != . ]; then openpkg_dev_package $1; fi
    if [ ".$2" != . ]; then openpkg_dev_branch  $2; fi
    if [ ".$3" != . ]; then openpkg_dev_execute $3; fi
    if [ ".$4" != . ]; then openpkg_dev_tools   $4; fi
    if [ ".$5" != . ]; then openpkg_dev_src     $5; fi
    if [ ".${P}" = . ]; then
        cd src
    else
        cd
    fi

    #   self destruction
    rm ${OPENPKG_WORK}/.bashrc
}

bash () {
    #   honor optional parameters [[[spec]ctag]exec]; use "" to specify a empty parameter and use it's default
    [ ".$1" != . ] && OPENPKG_SPEC="$1"
    [ ".$2" != . ] && OPENPKG_CTAG="$2"
    [ ".$3" != . ] && OPENPKG_EXEC="$3"
    [ ".$4" != . ] && OPENPKG_TRUN="$4"
    [ ".$5" != . ] && OPENPKG_SDIR="$5"

    #   enter working directory
    builtin cd "${OPENPKG_WORK}" || die "cannot cd to ${OPENPKG_WORK}"
    P="${OPENPKG_SPEC}"

    #   canonify OPENPKG_CTAG
    canonifybranch "${OPENPKG_CTAG}"
    B="${RV}"

    #   canonify OPENPKG_EXEC
    matchinstance "${OPENPKG_EXEC}" exact
    E="${RV}"

    #   we must have a E/bin/rpm somewhere
    if [ ".$E" = . ]; then
        matchinstance "${OPENPKG_CTAG}" any
        E="${RV}"
    fi

    #   create a .bashrc
                                     >.bashrc
    dumpfunc -i bashrcprolog        >>.bashrc
    dumpfunc    warn                >>.bashrc
    dumpfunc    error               >>.bashrc
    dumpfunc    makedir             >>.bashrc
    dumpfunc    uwhich              >>.bashrc
    dumpfunc    initmpx             >>.bashrc
    dumpfunc    openpkgi            >>.bashrc
    dumpfunc    openpkgE            >>.bashrc
    dumpfunc    openpkgT            >>.bashrc
    dumpfunc    openpkg             >>.bashrc
    dumpfunc    rpmi                >>.bashrc
    dumpfunc    rpmE                >>.bashrc
    dumpfunc    rpmT                >>.bashrc
    dumpfunc    rpm                 >>.bashrc
    dumpfunc    openpkg_dev_package >>.bashrc
    dumpfunc    openpkg_dev_branch  >>.bashrc
    dumpfunc    openpkg_dev_execute >>.bashrc
    dumpfunc    openpkg_dev_tools   >>.bashrc
    dumpfunc    openpkg_dev_srcdir  >>.bashrc
    dumpfunc    root                >>.bashrc
    dumpfunc    cvs                 >>.bashrc
    dumpfunc -i bashrcepilog        >>.bashrc

    #   make sure temporary area exists
    if [ ! -d ${OPENPKG_TEMP} ]; then
        makedir 700 ${OPENPKG_TEMP} || die "cannot create ${OPENPKG_TEMP}"
    fi

    #   find reasonable bash
    matchinstance "" any
    if [ -f "${RV}/bin/bash" ]; then
        bash="${RV}/bin/bash"
    elif [ -f "${RV}/lib/openpkg/bash" ]; then
        bash="${RV}/lib/openpkg/bash"
    elif [ -f "`uwhich bash`" ]; then
        bash="`uwhich bash`"
    else
        die "no \"bash\" found in OpenPKG instance"
    fi

    #   enter environment
    echo "++ entering OpenPKG development shell (mode: [1m${OPENPKG_MODE}[m)"
    ${bash} --rcfile .bashrc
    echo "++ leaving OpenPKG development shell"

    #   remove workdir if it is completely empty
    #package -query workdir ${OPENPKG_SPEC} >/dev/null FIXME
    #rmdir $workdir 2>/dev/null                        FIXME
}

list () {
    if [ ".$1" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="$*"
    fi
    locationid=`rpmE --eval %{_rpmfilename} | sed -e 's;^.*}-;;' -e 's;\.rpm$;;'`
    for name in $all; do
        name=`echo $name | sed -e 's;\.src\.rpm$;;' -e 's;\.nosrc\.rpm$;;'` # convenience
        ls -l ${OPENPKG_WORK}/pkg/src/$name* | ${EGREP} "/$name[^-]*-[^-]+-[^-]+\.(no)?src.rpm$"
        ls -l ${OPENPKG_WORK}/pkg/bin/$name* | ${EGREP} "/$name[^-]*-[^-]+-[^-]+\.[^-]+-[^-]+-${locationid}.rpm$"
    done
}

peek () {
    if [ ".$1" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="$*"
    fi
    for name in $all; do
        name=`echo $name | sed -e 's;\.src\.rpm$;;' -e 's;\.nosrc\.rpm$;;'` # convenience
        if [ ! -f "${OPENPKG_WORK}/$S/${name}/${name}.spec" ]; then
            warn "${OPENPKG_WORK}/$S/${name}/${name}.spec not found"
            continue
        fi
        template=`rpmE --eval '%{_rpmfilename}'`
        rpmfile=`rpmE -q --qf "${template}" --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
        rpmdir=`rpmE --eval '%{_rpmdir}'`
        if [ ! -f "$rpmdir/$rpmfile" ]; then
            die "binary RPM not found: $rpmdir/$rpmfile"
        fi
        echo "++ peeking at file list of binary RPM: $rpmfile"
        filter="sed -e 's;^;\\ \\ \\ ;'"
        tmpfile="${OPENPKG_TEMP}/${name}${GUID}"
        for configfile in `rpmE -qplc $rpmdir/$rpmfile`; do
            filter="$filter -e \"s;^ \\\\(.* ${configfile}\\\\)\$;C\\\\1;\""
        done
        for docfile in `rpmE -qpld $rpmdir/$rpmfile`; do
            IFS="$OIFS"
            filter="$filter -e \"s;^ \\\\(.* ${docfile}\\\\)\$;D\\\\1;\""
        done
        rpmE -qplv $rpmdir/$rpmfile | eval $filter
    done
}

diff () {
    #   the name comes from the "cvs diff" command. It is actually a
    #   "dry" run of release() without altering anything.
    release -dry "$@"
}

release () {
    #   honor optional parameters [-dry] [-m "message"] [-x]
    dry=0
    extra=0
    msg=""
    packages=""
    while [ ".$1" != . ]; do
        case ".$1" in
            .-dry ) dry=1 ;;
            .-x   ) extra=1 ;;
            .-m   ) [ ".$2"    = . ] && die "-m option requires argument"
                    [ ".$msg" != . ] && die "-m can only be specified once"
                    shift
                    msg="$1" ;;
                * ) packages="${packages} $1"
        esac
        shift
    done

    #   examine if environment supports colorized diff
    if [ ".`uwhich cvs-diff-colorize`" != . ]; then
        cvs_diff_colorize="cvs-diff-colorize"
    else
        cvs_diff_colorize="cat"
    fi

    #   sanity check
    [ ".${OPENPKG_SPEC:-$P}" != ".$P" ] && die "release requires OPENPKG_SPEC (=${OPENPKG_SPEC}) and P (=$P) being in sync"
    [ ".${OPENPKG_CTAG:-$B}" != ".$B" ] && die "release requires OPENPKG_CTAG (=${OPENPKG_CTAG}) and B (=$B) being in sync"
    [ ".${OPENPKG_EXEC:-$E}" != ".$E" ] && die "release requires OPENPKG_EXEC (=${OPENPKG_EXEC}) and E (=$E) being in sync"

    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi

    #   determine whether CVS commit messages can be bundled
    bundle=0
    names=0
    for name in $all; do
        names=`expr $names + 1`
    done
    if [ ".${msg}" != . -a $names -gt 2 ]; then
        bundle=1
        bundle_files=""
        bundle_srpms=""
        bundle_msg=""
        if [ -f "${msg}" ]; then
            bundle_msg="-F \"${msg}\""
        elif [ ".${msg}" = ".-" ]; then
            bundle_msg=""
        else
            bundle_msg="-m \"$msg\""
        fi
    fi

    for name in $all; do
        name=`echo $name | sed -e 's;\.src\.rpm$;;' -e 's;\.nosrc\.rpm$;;'` # convenience
        name=`echo ${name} | sed -e 's;-[^-][^-]*-[^-][^-]*$;;'`            # convenience
        echo "++ processing package ${name}"
        if [ ! -f "${OPENPKG_WORK}/$S/${name}/${name}.spec" ]; then
            die "${OPENPKG_WORK}/$S/${name}/${name}.spec not found"
        fi
        template=`rpmE --eval '%{_rpmfilename}' | sed -e 's;\..*$;;'`
        rpmfile=`rpmE -q --qf "${template}" --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`

        echo "++ cleaning up sources"
        rpmE --clean --nodeps ${OPENPKG_WORK}/$S/${name}/${name}.spec >/dev/null 2>&1 || true

        # developer section
        if [ ".${OPENPKG_MODE}" = .developer ]; then

            # compatiblity FIXME
            echo "${OPENPKG_DIST}" | ${EGREP} "/current/SRC/?$" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                warn "deprecated suffix /current/src in OPENPKG_DIST=${OPENPKG_DIST}."
                warn "This might come from your ~/.openpkg/dev.rc file. Please fix."
                warn "Three second penalty :-)"
                sleep 3
                OPENPKG_DIST=`echo ${OPENPKG_DIST} | sed -e 's;/$;;' -e 's;/current/SRC$;;'`
            fi
            if [ ".${OPENPKG_WILD}" != . ]; then
                warn "deprecated use of OPENPKG_WILD. Do not use it."
                warn "Please list all instances in OPENPKG_INST (space delimited)."
                warn "This might come from your ~/.openpkg/dev.rc file. Please fix."
                warn "Three second penalty :-)"
                sleep 3
            fi

            echo "++ determine [no]src extension"
            ext=""
            [ -f "${OPENPKG_WORK}/pkg/src/${rpmfile}.src.rpm" ]   && ext="src.rpm"
            [ -f "${OPENPKG_WORK}/pkg/src/${rpmfile}.nosrc.rpm" ] && ext="nosrc.rpm"
            [ ".$ext" = . ] && die "${OPENPKG_WORK}/pkg/src/${rpmfile}.[no]src.rpm not found"

            echo "++ determine old version (if any)"
            tmpfile="${OPENPKG_TEMP}/${name}${GUID}"
            cvs_stat=`builtin cd ${OPENPKG_WORK}; cvs -d ${OPENPKG_REPO} stat $S/${name}/${name}.spec 2>/dev/null | grep 'Repository revision:'`
            cvs_rev=`echo "${cvs_stat}" | ${AWK} '/Repository revision:/ { print $3; }'`
            cvs_dead=`echo "${cvs_stat}" | grep '/Attic/'`
            if [ ".${cvs_dead}" != . ]; then
                cvs_rev="No"
            fi
            if [ ".${cvs_rev}" != .No ]; then
                cvs -d ${OPENPKG_REPO} co -p -r${cvs_rev} openpkg-src/${name}/${name}.spec >${tmpfile} 2>/dev/null
                rpm_version_old=`rpmE -q --qf "%{VERSION}" --specfile ${tmpfile} 2>/dev/null`
                rpm_release_old=`rpmE -q --qf "%{RELEASE}" --specfile ${tmpfile} 2>/dev/null`
                rm -f ${tmpfile}
                [ ".${rpm_version_old}" = . ] && die "cannot determine old version number"
                [ ".${rpm_release_old}" = . ] && die "cannot determine old release number"
            fi

            echo "++ determine new version"
            rpm_version_new=`rpmE -q --qf "%{VERSION}" --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
            rpm_release_new=`rpmE -q --qf "%{RELEASE}" --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
            rpm_name=`rpmE -q --qf "%{NAME}" --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
            rpm_summary=`rpmE -q --qf "%{SUMMARY}" --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
            [ ".${rpm_version_new}" = . ] && die "cannot determine new version number"
            [ ".${rpm_release_new}" = . ] && die "cannot determine new release number"

            echo "++ determine branch and related release number and upload suffix"
            if [ ".$B" = .HEAD ]; then
                BRANCH="HEAD"
            else
                BRANCH=`echo "$B" | sed -e 's;^OPENPKG_;;' -e 's;_STABLE$;;' -e 's;SOLID$;;' -e 's;^HEAD$;;' -e 's;_;;g'`
                OPMAJORV=`echo ${BRANCH} | cut -c 1`
                OPMINORV=`echo ${BRANCH} | cut -c 2`
                if [ ".${OPMINORV}" != . ]; then
                    BRANCH="SOLID"
                elif [ ".${OPMAJORV}" != . ]; then
                    BRANCH="STABLE"
                else
                    BRANCH=""
                fi
            fi
            rpm_release_req=""
            upload=""
            if [ ".${BRANCH}" = .HEAD ]; then
                rpm_release_req=`date '+%Y%m%d'`
                upload="current/SRC/"
            fi
            if [ ".${BRANCH}" = .STABLE ]; then
                rpm_release_req="${OPMAJORV}.`date '+%Y%m%d'`"
                #upload="stable/${OPMAJORV}/SRC/"
                upload="stable/SRC/"
            fi
            if [ ".${BRANCH}" = .SOLID ]; then
                update=`echo ${rpm_release_old} | sed -e 's;^.*\.;;'`
                if [ ".${update}" = . ]; then
                    update=0
                else
                    update=`expr ${update} + 1`
                fi
                rpm_release_req="${OPMAJORV}.${OPMINORV}.${update}"
                upload="release/${OPMAJORV}.${OPMINORV}/UPD/00UPLOAD/"
            fi

            echo "++ checking release number"
            if [ ".$rpm_release_req" = . ]; then
                die "cannot predict required release number for branch $B"
            fi
            if [ ".$rpm_release_new" != ".$rpm_release_req" ]; then
                die "release number ${rpm_release_new} does not match expectation ${rpm_release_req}"
            fi

            echo "++ checking upload suffix"
            if [ ".$upload" = . ]; then
                die "cannot determine upload suffix for branch $B"
            fi

            if [ ".${BRANCH}" = .HEAD ]; then
                echo "++ linting ${name}.spec (package specification)"
                openpkgT lint-spec ${OPENPKG_WORK}/$S/${name}/${name}.spec
                if [ $? -gt 0 ]; then
                    die "unacceptable specification: ${OPENPKG_WORK}/$S/${name}/${name}.spec"
                fi
                if [ -f rc.${name} ]; then
                    echo "++ linting rc.${name} (run-command script)"
                    openpkgT lint-rc ${OPENPKG_WORK}/$S/${name}/rc.${name}
                    if [ $? -gt 0 ]; then
                        die "unacceptable run commands: ${OPENPKG_WORK}/$S/${name}/rc.${name}"
                    fi
                fi
                if [ -f fsl.${name} ]; then
                    echo "++ linting fsl.${name} (OSSP fsl configuration)"
                    openpkgT lint-fsl ${OPENPKG_WORK}/$S/${name}/fsl.${name}
                    if [ $? -gt 0 ]; then
                        die "unacceptable fsl configuration: ${OPENPKG_WORK}/$S/${name}/fsl.${name}"
                    fi
                fi
            fi

            if [ ".${BRANCH}" = .HEAD -a ".$extra" = .1 ]; then
                bin_template=`rpmE --eval '%{_rpmfilename}'`
                bin_rpmfile=`rpmE -q --qf "${bin_template}" \
                             --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
                bin_rpmdir=`rpmE --eval '%{_rpmdir}'`
                if [ -f "$bin_rpmdir/$bin_rpmfile" ]; then
                    echo "++ linting ${bin_rpmfile} (binary package)"
                    openpkgT lint-rpm --check=layout,attrib $bin_rpmdir/$bin_rpmfile
                    if [ $? -gt 0 ]; then
                        die "unacceptable specification: $bin_rpmdir/$bin_rpmfile"
                    fi
                fi
            fi

            echo "++ releasing ${rpmfile}.${ext} to OpenPKG distribution area ${upload}"
            if [ ${bundle} -eq 1 ]; then
                bundle_srpms="${bundle_srpms} ${OPENPKG_WORK}/pkg/src/${rpmfile}.${ext}"
            elif [ ${dry} -eq 0 ]; then
                chmod 664 ${OPENPKG_WORK}/pkg/src/${rpmfile}.${ext}
                scp -o "Compression no" -o "Cipher blowfish" ${OPENPKG_WORK}/pkg/src/${rpmfile}.${ext} ${OPENPKG_DIST}/${upload}
            else
                echo chmod 664 ${OPENPKG_WORK}/pkg/src/${rpmfile}.${ext}
                echo scp -o "Compression no" -o "Cipher blowfish" ${OPENPKG_WORK}/pkg/src/${rpmfile}.${ext} ${OPENPKG_DIST}/${upload}
            fi

            if [ ".${name}" = .openpkg ]; then
                if [ -f "${OPENPKG_WORK}/pkg/src/${rpmfile}.src.sh" ]; then
                    echo "++ releasing ${rpmfile}.src.sh to OpenPKG distribution area ${upload}"
                    if [ ${dry} -eq 0 ]; then
                        chmod 664 ${OPENPKG_WORK}/pkg/src/${rpmfile}.src.sh
                        scp -o "Compression no" -o "Cipher blowfish" ${OPENPKG_WORK}/pkg/src/${rpmfile}.src.sh ${OPENPKG_DIST}/${upload}
                    else
                        echo chmod 664 ${OPENPKG_WORK}/pkg/src/${rpmfile}.src.sh
                        echo scp -o "Compression no" -o "Cipher blowfish" ${OPENPKG_WORK}/pkg/src/${rpmfile}.src.sh ${OPENPKG_DIST}/${upload}
                    fi
                else
                    die "required file ${OPENPKG_WORK}/pkg/src/${rpmfile}.src.sh missing"
                fi
            fi

            echo "++ determining commit message"
            if [ ".$msg" = . ]; then
                if [   ".${rpm_version_old}" = . \
                    -a ".${rpm_version_new}" != . \
                    -a ".${rpm_name}" != . \
                    -a ".${rpm_summary}" != . ]; then
                     msg="new package: ${rpm_name} ${rpm_version_new} (${rpm_summary})"
                elif [ ".${rpm_version_old}" != . \
                    -a ".${rpm_version_new}" != . \
                    -a ".${rpm_version_old}" != ".${rpm_version_new}" ]; then
                     msg="upgrading package: ${rpm_name} ${rpm_version_old} -> ${rpm_version_new}"
                elif [ ".${rpm_version_old}" != . \
                    -a ".${rpm_version_new}" != . \
                    -a ".${rpm_version_old}" = ".${rpm_version_new}" \
                    -a ".${rpm_release_old}" != ".${rpm_release_new}" ]; then
                     msg="modifying package: ${rpm_name}-${rpm_version_new} ${rpm_release_old} -> ${rpm_release_new}"
                elif [ ".${rpm_version_old}" != . \
                    -a ".${rpm_version_new}" != . \
                    -a ".${rpm_version_old}" = ".${rpm_version_new}" \
                    -a ".${rpm_release_old}" = ".${rpm_release_new}" ]; then
                     msg="modifying package: ${rpm_name}-${rpm_version_new} ${rpm_release_old} again"
                fi
                if [ ".${msg}" = . ]; then
                    die "unable to determine type of source changes. Please specify commit message"
                fi
            fi

            echo "++ determining commit file list"
            files="$S/${name}"

            echo "++ comitting changes to CVS repository"
            builtin cd "${OPENPKG_WORK}" || die "cannot cd to ${OPENPKG_WORK}"
            if [ ".${msg}" != . -a -f "${msg}" ]; then
                if [ ${bundle} -eq 1 ]; then
                    bundle_files="${bundle_files} ${files}"
                elif [ ${dry} -eq 0 ]; then
                    cvs -d ${OPENPKG_REPO} commit -F "${msg}" $files
                else
                    echo cvs commit -F "${msg}" $files
                    cvs -d ${OPENPKG_REPO} diff $files | ${cvs_diff_colorize}
                fi
            elif [ ".${msg}" = ".-" ]; then
                if [ ${bundle} -eq 1 ]; then
                    bundle_files="${bundle_files} ${files}"
                elif [ ${dry} -eq 0 ]; then
                    cvs -d ${OPENPKG_REPO} commit $files
                else
                    echo cvs commit $files
                    cvs -d ${OPENPKG_REPO} diff $files | ${cvs_diff_colorize}
                fi
            else
                if [ ${bundle} -eq 1 ]; then
                    bundle_files="${bundle_files} ${files}"
                elif [ ${dry} -eq 0 ]; then
                    cvs -d ${OPENPKG_REPO} commit -m "${msg}" $files
                else
                    echo cvs commit -m "${msg}" $files
                    cvs -d ${OPENPKG_REPO} diff $files | ${cvs_diff_colorize}
                fi
            fi

        # contributor section
        else
            echo "++ determining contributor identification"
            thisdate=`${OPENPKG_EXEC}/lib/openpkg/shtool echo -e '%D-%m-%Y'`
            username=`${OPENPKG_EXEC}/lib/openpkg/shtool echo -e '%u'`
            hostname=`${OPENPKG_EXEC}/lib/openpkg/shtool echo -e '%h%d'`
            realname=`(getent passwd; cat /etc/passwd; ypcat passwd; nidump passwd .) 2>/dev/null |\
                      grep "^${username}:" | ${AWK} -F: '{ print $5; }'`
            if [ ".$realname" = . ]; then
                realname="NN"
            fi

            echo "++ generating patch file (${OPENPKG_WORK}/${name}.diff)"
            ( echo "From: $realname <$username@$hostname>"
              echo "Date: $thisdate"
              if [ ".${msg}" = . ]; then
                  msg="unknown contribution"
              fi
              if [ -f "${msg}" ]; then
                  msg="`cat ${msg}`"
              fi
              echo "Subject: ${msg}"
              echo ""
              ( builtin cd ${OPENPKG_WORK}; cvs -d ${OPENPKG_REPO} diff -N -u3 $S/${name} )
            ) >${OPENPKG_WORK}/${name}.diff

            echo "++ uploading patch file to OpenPKG contribution area"
            curlE=`rpmE --eval '%{l_curl}'`
            if [ ${dry} -eq 0 ]; then
                $curlE --upload-file ${OPENPKG_WORK}/${name}.diff ${OPENPKG_DIST}/${name}.diff
            else
                echo $curlE --upload-file ${OPENPKG_WORK}/${name}.diff ${OPENPKG_DIST}/${name}.diff
                cat ${OPENPKG_WORK}/${name}.diff | ${cvs_diff_colorize}
            fi
        fi
    done

    #   lazy execution of bundled CVS commit
    if [ ${bundle} -eq 1 ]; then
        builtin cd "${OPENPKG_WORK}" || die "cannot cd to ${OPENPKG_WORK}"
        if [ ${dry} -eq 0 ]; then
            chmod 664 ${bundle_srpms}
            scp -o "Compression no" -o "Cipher blowfish" ${bundle_srpms} ${OPENPKG_DIST}/${upload}
            eval "cvs -d ${OPENPKG_REPO} commit ${bundle_msg} ${bundle_files}"
        else
            echo scp -o "Compression no" -o "Cipher blowfish" ${bundle_srpms} ${OPENPKG_DIST}/${upload}
            echo "cvs -d ${OPENPKG_REPO} commit ${bundle_msg} ${bundle_files}"
            cvs -d ${OPENPKG_REPO} diff ${bundle_files} | ${cvs_diff_colorize}
        fi
    fi
}

kill () {
    case ${OPENPKG_WORK} in
        /?* ) ;;
          * ) die "working area ${OPENPKG_WORK} seems to be strange" ;;
    esac
    if [ -d ${OPENPKG_WORK} ]; then
        echo "++ killing working area ${OPENPKG_WORK}"
        rm -rf ${OPENPKG_WORK} >/dev/null 2>&1 || true
    fi
}

clean () {
    case ${OPENPKG_TEMP} in
        /?* ) ;;
          * ) die "temporary area ${OPENPKG_TEMP} seems to be strange" ;;
    esac
    if [ -d ${OPENPKG_TEMP} ]; then
        echo "++ cleaning temporary area ${OPENPKG_TEMP}"
        rm -rf ${OPENPKG_TEMP}/* ${OPENPKG_TEMP}/.[a-z]* >/dev/null 2>&1 || true
    fi
}

new () {
    #   honor optional parameters [-dry]
    dry=0
    packages=""
    while [ ".$1" != . ]; do
        case ".$1" in
            .-dry ) dry=1 ;;
                * ) packages="${packages} $1"
        esac
        shift
    done

    if [ ".${packages}" = . ]; then
        die "package cannot be identified. Parameter is mandatory."
    else
        all="${packages}"
    fi

    for name in $all; do
        if [ -d "${OPENPKG_WORK}/$S/${name}" ]; then
            die "directory ${OPENPKG_WORK}/$S/${name} already exists -- remove first"
        fi

        echo "++ creating openpkg-src/${name}/ CVS area"
        if [ ${dry} -eq 0 ]; then
            makedir 755 ${OPENPKG_WORK}/$S/${name} || die "cannot create ${OPENPKG_WORK}/$S/${name}"
            if [ ".${OPENPKG_MODE}" = .developer ]; then
                ( builtin cd ${OPENPKG_WORK}/src && cvs -d ${OPENPKG_REPO} add ${name} )
            fi
        else
            echo "makedir 755 ${OPENPKG_WORK}/$S/${name}"
            if [ ".${OPENPKG_MODE}" = .developer ]; then
                echo "cd ${OPENPKG_WORK}/src && cvs -d ${OPENPKG_REPO} add ${name}"
            fi
        fi

        echo "++ creating openpkg-src/${name}/${name}.spec skeleton"
        year=`date '+%Y'`
        if [ ${dry} -eq 0 ]; then
            ( echo "##"
              echo "##  ${name}.spec -- OpenPKG RPM Specification"
              echo "##  Copyright (c) 2000-${year} The OpenPKG Project <http://www.openpkg.org/>"
              echo "##  Copyright (c) 2000-${year} Ralf S. Engelschall <rse@engelschall.com>"
              echo "##  Copyright (c) 2000-${year} Cable & Wireless <http://www.cw.com/>"
              echo "##"
              echo "##  Permission to use, copy, modify, and distribute this software for"
              echo "##  any purpose with or without fee is hereby granted, provided that"
              echo "##  the above copyright notice and this permission notice appear in all"
              echo "##  copies."
              echo "##"
              echo "##  THIS SOFTWARE IS PROVIDED \`\`AS IS'' AND ANY EXPRESSED OR IMPLIED"
              echo "##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF"
              echo "##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED."
              echo "##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR"
              echo "##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,"
              echo "##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT"
              echo "##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF"
              echo "##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND"
              echo "##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,"
              echo "##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT"
              echo "##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF"
              echo "##  SUCH DAMAGE."
              echo "##"
              echo ""
              echo "#   package information"
              echo "Name:         ${name}"
              echo "Summary:      Badly Packaged Program"
              echo "URL:          http://www.${name}.org/"
              echo "Vendor:       John Doe"
              echo "Packager:     The OpenPKG Project"
              echo "Distribution: OpenPKG"
              echo "Class:        EVAL"
              echo "Group:        Unknown"
              echo "License:      DoePL"
              echo "Version:      7.42"
              echo "Release:      `date '+%Y%m%d'`"
              echo ""
              echo "#   list of sources"
              echo "Source0:      ftp://ftp.${name}.org/pub/${name}/${name}-%{version}.tar.gz"
              echo ""
              echo "#   build information"
              echo "Prefix:       %{l_prefix}"
              echo "BuildRoot:    %{l_buildroot}"
              echo "BuildPreReq:  OpenPKG, openpkg >= 20040130"
              echo "PreReq:       OpenPKG, openpkg >= 20040130"
              echo "AutoReq:      no"
              echo "AutoReqProv:  no"
              echo ""
              echo "%description"
              echo "    ${name} is the most badly packaged program in the OpenPKG world."
              echo ""
              echo "%track"
              echo "    prog ${name} = {"
              echo "        version   = %{version}"
              echo "        url       = ftp://ftp.${name}.org/pub/${name}/"
              echo "        regex     = ${name}-(__VER__)\\.tar\\.gz"
              echo "    }"
              echo ""
              echo "%prep"
              echo "    %setup -q"
              echo ""
              echo "%build"
              echo "    CC=\"%{l_cc}\" \\"
              echo "    CXX=\"%{l_cxx}\" \\"
              echo "    CFLAGS=\"%{l_cflags -O}\" \\"
              echo "    CXXFLAGS=\"%{l_cxxflags -O}\" \\"
              echo "    CPPFLAGS=\"%{l_cppflags}\" \\"
              echo "    LDFLAGS=\"%{l_ldflags}\" \\"
              echo "    ./configure \\"
              echo "        --prefix=%{l_prefix}"
              echo "    %{l_make} %{l_mflags -O}"
              echo ""
              echo "%install"
              echo "    rm -rf \$RPM_BUILD_ROOT"
              echo "    %{l_make} %{l_mflags} install AM_MAKEFLAGS=\"DESTDIR=\$RPM_BUILD_ROOT\""
              echo "    strip \$RPM_BUILD_ROOT%{l_prefix}/bin/* >/dev/null 2>&1 || true"
              echo "    %{l_rpmtool} files -v -ofiles -r\$RPM_BUILD_ROOT %{l_files_std}"
              echo ""
              echo "%files -f files"
              echo ""
              echo "%clean"
              echo "    rm -rf \$RPM_BUILD_ROOT"
              echo ""
            ) >${OPENPKG_WORK}/$S/${name}/${name}.spec
            if [ ".${OPENPKG_MODE}" = .developer ]; then
                ( builtin cd ${OPENPKG_WORK}/$S/${name} && cvs -d ${OPENPKG_REPO} add ${name}.spec )
            fi
        else
            echo "creating ${OPENPKG_WORK}/$S/${name}/${name}.spec"
            if [ ".${OPENPKG_MODE}" = .developer ]; then
                echo "cd ${OPENPKG_WORK}/$S/${name} && cvs -d ${OPENPKG_REPO} add ${name}.spec"
            fi
        fi
    done
}

install () {
    force=0
    packages=""
    while [ ".$1" != . ]; do
        case ".$1" in
            .-f ) force=1 ;;
              * ) packages="${packages} $1"
        esac
        shift
    done
    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi
    for name in $all; do
        echo "++ installing ${name}"
        if [ ! -f "${OPENPKG_WORK}/$S/${name}/${name}.spec" ]; then
            warn "${OPENPKG_WORK}/$S/${name}/${name}.spec not found"
            continue
        fi
        package=`rpmE -q --qf "%{NAME}-%{VERSION}-%{RELEASE}" \
                 --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
        template=`rpmE --eval '%{_rpmfilename}'`
        rpmfile=`rpmE -q --qf "${template}" \
                 --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
        rpmdir=`rpmE --eval '%{_rpmdir}'`
        if [ ! -f "$rpmdir/$rpmfile" ]; then
            build ${name}
        fi
        if [ ".$HOSTNAME" = . ]; then
            HOSTNAME=`hostname`
        fi
        initmpx $E
        if [ $force -ne 0 ]; then
            echo "++ installing $package ($rpmfile) forced"
            ssh -t -x "root@${HOSTNAME}" ${RPMCMD} -Uvh "$rpmdir/$rpmfile" --force --nodeps
        else
            rpmE -q $package >/dev/null 2>&1 && continue
            echo "++ installing $package ($rpmfile)"
            ssh -t -x "root@${HOSTNAME}" ${RPMCMD} -Uvh "$rpmdir/$rpmfile"
        fi
        rpmE -q $package >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            warn "failed to install $package"
        fi
        echo "++ cleaning up sources"
        rpmE --clean --nodeps ${OPENPKG_WORK}/$S/${name}/${name}.spec >/dev/null 2>&1 || true
    done
}

build () {
    force=0
    packages=""
    defall=""
    defpkg=""
    while [ ".$1" != . ]; do
        case ".$1" in
            .-D ) shift
                  opt="$1"
                  pkg=""; echo "${opt}" | ${EGREP} '::' >/dev/null 2>&1 && pkg="`echo ${opt} | sed -e 's;::.*$;;'`"
                  opt="`echo ${opt} | sed -e 's;^.*::;;'`"
                  echo "${opt}" | ${EGREP} '^with_' >/dev/null 2>&1 || opt="with_${opt}"
                  echo "${opt}" | ${EGREP} '='      >/dev/null 2>&1 || opt="${opt}=yes"
                  if [ ".${pkg}" = . ]; then
                      defall="${defall} ${opt}"
                  else
                      defpkg="${defpkg} ${pkg}::${opt}"
                  fi
                  ;;
            .-f ) force=1 ;;
              * ) packages="${packages} $1"
        esac
        shift
    done
    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi
    for name in $all; do
        opt="`echo ${defpkg} | tr ' ' '\n' | ${EGREP} ^$name:: | sed 's;^.*::;;' | tr '\n' ' '`"
        opt="`echo ${opt} $defall`"
        echo "++ building ${name} ${opt}"
        if [ ! -f "${OPENPKG_WORK}/$S/${name}/${name}.spec" ]; then
            warn "${OPENPKG_WORK}/$S/${name}/${name}.spec not found"
            continue
        fi
        package=`rpmE -q --qf "%{NAME}-%{VERSION}-%{RELEASE}" \
                 --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
        template=`rpmE --eval '%{_rpmfilename}'`
        rpmfile=`rpmE -q --qf "${template}" \
                 --specfile ${OPENPKG_WORK}/$S/${name}/${name}.spec`
        rpmdir=`rpmE --eval '%{_rpmdir}'`
        if [ ! -f "$rpmdir/$rpmfile" -o $force -ne 0 ]; then
            echo "++ building $package ($rpmfile)"
            opt="`echo ${opt} | tr ' ' '\n' | ${EGREP} . | sed -e 's;^;--define \";' -e 's;=; ;' -e 's;$;\";' | tr '\n' ' '`"
            (builtin cd ${OPENPKG_WORK}/$S/${name} && openpkgT fetch ${name}.spec)
            (builtin cd ${OPENPKG_WORK}/$S/${name} && eval "rpmE -ba ${opt} ${name}.spec")
            if [ ! -f "$rpmdir/$rpmfile" ]; then
                warn "failed to build $package ($rpmfile)"
                continue
            fi
        fi
    done
}

search_for () {
    URL=`echo $2 | sed -e 's| *$||'`
    PKG=`$E/lib/openpkg/curl -l -s $URL |\
         sed -e 's|[^-a-zA-Z0-9_\.]||g' |\
         grep "^$1" | sort`
    if [ ".$PKG" != . ]; then
        for pkg in $PKG; do
            NAME=`echo "$pkg" | sed -e 's;\.src\.rpm$;;' -e 's;\.src\.sh$;;'`
            NAME=`echo "$NAME ........................................" | cut -c-35`
            DLOC=`echo "$pkg" | sed -e "s;^;$URL;"`
            echo "$NAME $DLOC"
        done
    fi
}

search () {
    #   we must have a E/bin/rpm somewhere
    if [ ".$E" = . ]; then
        matchinstance "${OPENPKG_CTAG}" any
        E="${RV}"
    fi

    if [ ".$1" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="$*"
    fi
    for i in $all; do
        search_for $i ftp://ftp.openpkg.org/current/SRC/
        search_for $i ftp://ftp.openpkg.org/stable/SRC/
        search_for $i ftp://ftp.openpkg.org/release/2.0/UPD/
        search_for $i ftp://ftp.openpkg.org/release/2.0/SRC/
        search_for $i ftp://ftp.openpkg.org/release/2.0/SRC/PLUS/
        search_for $i ftp://ftp.openpkg.org/release/1.3/UPD/
        search_for $i ftp://ftp.openpkg.org/release/1.3/SRC/
        search_for $i ftp://ftp.openpkg.org/release/1.3/SRC/PLUS/
        search_for $i ftp://ftp.openpkg.org/release/1.2/UPD/
        search_for $i ftp://ftp.openpkg.org/release/1.2/SRC/
        search_for $i ftp://ftp.openpkg.org/release/1.2/SRC/PLUS/
        search_for $i ftp://ftp.openpkg.org/release/1.1/UPD/
        search_for $i ftp://ftp.openpkg.org/release/1.1/SRC/
        search_for $i ftp://ftp.openpkg.org/release/1.1/SRC/PLUS/
        search_for $i ftp://ftp.openpkg.org/release/1.0/UPD/
        search_for $i ftp://ftp.openpkg.org/release/1.0/SRC/
    done
}

lint () {
    force=0
    packages=""
    extra=0
    while [ ".$1" != . ]; do
        case ".$1" in
            .-f ) force=1 ;;
            .-x ) extra=1 ;;
              * ) packages="${packages} $1"
        esac
        shift
    done
    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi
    for name in $all; do
        echo "$name" | ${EGREP} "^[./]+" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "$name" | ${EGREP} "/" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                name="${OPENPKG_WORK}/$S/$name"
            else
                name=`echo $name | sed -e 's;\.spec$;;'` # convenience
                name="${OPENPKG_WORK}/$S/$name/$name.spec"
            fi
        fi
        if [ -d $name ]; then
            files="`find $name -type f -name '*.spec' -print | sort`"
        else
            files="$name"
        fi
        for file in $files; do
            name=`echo ${file} | sed -e 's;^.*/;;'`
            path=`echo ${file} | sed -e "s;/$name\$;;"`
            name=`echo ${name} | sed -e 's;\.spec$;;'`

            echo "++ linting $file (package specification)"
            openpkgT lint-spec $file
            if [ $? -gt 0 -a $force -ne 1 ]; then
                die "unacceptable specification"
            fi

            if [ -f ${path}/rc.${name} ]; then
                echo "++ linting ${path}/rc.${name} (run-command script)"
                openpkgT lint-rc ${path}/rc.${name}
                if [ $? -gt 0 -a $force -ne 1 ]; then
                    die "unacceptable run commands: ${path}/rc.${name}"
                fi
            fi

            if [ -f ${path}/fsl.${name} ]; then
                echo "++ linting ${path}/fsl.${name} (OSSP fsl configuration)"
                openpkgT lint-fsl ${path}/fsl.${name}
                if [ $? -gt 0 -a $force -ne 1 ]; then
                    die "unacceptable fsl configuration: ${path}/fsl.${name}"
                fi
            fi

            if [ ".$extra" = .1 ]; then
                bin_template=`rpmE --eval '%{_rpmfilename}'`
                bin_rpmfile=`rpmE -q --qf "${bin_template}" --specfile $file`
                bin_rpmdir=`HOME=${OPENPKG_WORK} rpmE --eval '%{_rpmdir}'`
                if [ -f "$bin_rpmdir/$bin_rpmfile" ]; then
                    echo "++ linting ${bin_rpmdir}/${bin_rpmfile} (binary package)"
                    echo openpkgT lint-rpm --check=layout,attrib $bin_rpmdir/$bin_rpmfile
                    openpkgT lint-rpm --check=layout,attrib $bin_rpmdir/$bin_rpmfile
                    if [ $? -gt 0 -a $force -ne 1 ]; then
                        die "unacceptable specification: $bin_rpmdir/$bin_rpmfile"
                    fi
                fi
            fi
        done
    done
}

testit () {
    # FIXME consider test() [shell function] vs. test(1) [builtin] vs. rpm [non-OpenPKG] --test
    rpmE --help | grep -- --test >/dev/null && echo "${RPMCMD:-rpm} does not support --test" # FIXME s/echo/die/
    die "testit not yet implemented. Developer: consider testit vs. test(1)"
}

vcheck () {
    # compatiblity FIXME
    die "vcheck is deprecated. Use track."
}

track () {
    rpmT --help | grep -- --track >/dev/null || die "${RPMCMD:-rpm} does not support --track"
    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi
    for name in $all; do
        rpmT --track ${name}.spec
    done
}

run () {
    #   the last parameter is the command to execute
    force=0
    dry=0
    packages=""
    while [ ".$2" != . ]; do
        case ".$1" in
            .-f   ) force=1 ;;
            .-dry ) dry=1 ;;
                * ) packages="${packages} $1"
        esac
        shift
    done

    execute="$1"
    if [ ".${execute}" = . ]; then
        die "command for execution missing"
    fi

    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi

    for name in $all; do
        echo "$name" | ${EGREP} "^[./]+" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "$name" | ${EGREP} "/" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                name="${OPENPKG_WORK}/$S/$name"
            else
                name=`echo $name | sed -e 's;\.spec$;;'` # convenience
                name="${OPENPKG_WORK}/$S/$name/$name.spec"
            fi
        fi
        if [ -d $name ]; then
            files="`find $name -type f -name '*.spec' -print | sort`"
        else
            files="$name"
        fi
        for file in $files; do
            echo "++ Running command for package specification: $file"
            tmpfile="${OPENPKG_TEMP}/${GUID}"
            ${AWK} -vexecute="$execute" -vfile="$file" <${file} >${tmpfile} '
                BEGIN { flag=0 }
                /^%prep$/ { flag=1 }
                /^$/ { if (flag == 1) print "    "execute" #"file; flag=0 }
                // { print $0 }'
            if [ ${dry} -eq 0 ]; then
                rpmE -bp ${tmpfile} --nodeps
                rc=$?
                rpmE --clean ${tmpfile} --nodeps
            else
                sed <${tmpfile} -n -e '/^%prep$/,/^$/p'
                rc=$?
            fi
            rm ${tmpfile}
            if [ $rc -gt 0 -a $force -ne 1 ]; then
                die "command in %prep section of ${tmpfile} returned error"
            fi
        done
    done
}

fetch () {
    cleana=0
    cleans=0
    force=0
    packages=""
    while [ ".$1" != . ]; do
        case ".$1" in
           .-a ) cleana=1 ;;
           .-s ) cleans=1 ;;
           .-f ) force=1 ;;
             * ) packages="${packages} $1"
        esac
        shift
    done

    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi

    for name in $all; do
        echo "$name" | ${EGREP} "^[./]+" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "$name" | ${EGREP} "/" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                name="${OPENPKG_WORK}/$S/$name"
            else
                name=`echo $name | sed -e 's;\.spec$;;'` # convenience
                name="${OPENPKG_WORK}/$S/$name/$name.spec"
            fi
        fi
        if [ -d $name ]; then
            files="`find $name -type f -name '*.spec' -print | sort`"
        else
            files="$name"
        fi
        for file in $files; do
            if [ $cleana -ne 0 ]; then
                spec=`echo ${file} | sed -e 's;^.*/;;' -e 's;\.spec$;;'`
                echo "++ Cleaning out all dst files for $spec"
                for s in `builtin cd ${OPENPKG_WORK}/dst/${spec}/ && echo *`; do
                    f="${OPENPKG_WORK}/dst/${spec}/${s}"
                    [ -f "${f}" ] && rm "${f}"
                    [ -e "${f}" -o -L "${f}" ] && die "Problem wiping out ${f}"
                done
            fi
            if [ $cleans -ne 0 ]; then
                spec=`echo ${file} | sed -e 's;^.*/;;' -e 's;\.spec$;;'`
                echo "++ Cleaning out selected dst files for $spec"
                for s in `builtin cd ${OPENPKG_WORK}/$S/${spec}/ && echo *`; do
                    f="${OPENPKG_WORK}/dst/${spec}/${s}"
                    [ -f "${f}" ] && rm "${f}"
                    [ -e "${f}" -o -L "${f}" ] && die "Problem wiping out ${f}"
                done
            fi
            echo "++ Fetching files for package specification: $file"
            openpkgT fetch ${file}
            if [ $? -gt 0 -a $force -ne 1 ]; then
                die "fetching sources for ${file} failed"
            fi
        done
    done
}

vim () {
    force=0
    packages=""
    while [ ".$1" != . ]; do
        case ".$1" in
           .-f ) force=1 ;;
             * ) packages="${packages} $1"
        esac
        shift
    done

    if [ ".${packages}" = . ]; then
        if [ ".$P" = . ]; then
            die "package cannot be identified. Neither parameter given nor \$P set."
        fi
        all="$P"
    else
        all="${packages}"
    fi

    for name in $all; do
        echo "$name" | ${EGREP} "^[./]+" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "$name" | ${EGREP} "/" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                name="${OPENPKG_WORK}/$S/$name"
            else
                name=`echo $name | sed -e 's;\.spec$;;'` # convenience
                name="${OPENPKG_WORK}/$S/$name/$name.spec"
            fi
        fi
        if [ -d $name ]; then
            files="`find $name -type f -name '*.spec' -print | sort`"
        else
            files="$name"
        fi
        for file in $files; do
            TERM=${TERM}-color command vim "${file}"
            if [ ".${OPENPKG_MODE}" = .developer -a ".${B}" = .HEAD -o $force -ne 0 ]; then
                openpkgT lint-spec ${file}
                if [ $? -gt 0 ]; then
                    die "unacceptable specification: ${file}"
                fi
            fi
        done
    done
}

help () {
    if [ -d ${OPENPKG_WORK} ]; then
        out="tee ${OPENPKG_WORK}/00README"
    else
        out="cat"
    fi
    sed -e 's;^ *;;' <<EOF | $out

        OpenPKG Development Area [created by $0]

        --------------------------------------------------------------------------------------------------------
        \$ openpkg dev bash    # [[[spec]ctag]exec]            enter development environment
        \$ openpkg dev build   # [[spec]...] [-f] [-D with...] build a package (forcibly) (with option)
        \$ openpkg dev branch  # [ctag]                        switch to a different branch
        \$ openpkg dev clean   #                               clean the temporary area
        \$ openpkg dev diff    #                               alias for "release -dry"
        \$ openpkg dev execute # [exec]                        execute command on instance closest to branch
        \$ openpkg dev fetch   # [[spec]...] [-s] [-a]         fetch sources; clean selected or all dst files
        \$ openpkg dev help    #                               display this command summary and update 00README
        \$ openpkg dev install # [[spec]...] [-f]              install a package (forcibly), build if required
        \$ openpkg dev kill    #                               kill the environment
        \$ openpkg dev lint    # [spec|file|path]...] [-f]     lint package specification
        \$ openpkg dev list    # [[spec]...]                   list source and binary packages
        \$ openpkg dev new     # spec... [-dry]                create new package from scratch
        \$ openpkg dev package # [spec]                        switch to a different package
        \$ openpkg dev peek    # [[spec]...]                   peek at file list of binary package
        \$ openpkg dev release # [[spec]...] [-dry] [-m "msg"] release package changes to master server
        \$ openpkg dev run     # [[spec]...] [-dry] [-f] cmd   append cmd to %prep for spec and run rpm -bp
        \$ openpkg dev search  # [[spec]...]                   search a package on master server
        \$ openpkg dev setup   # [[[spec]ctag]exec]            initial setup of openpkg dev working environment
        \$ openpkg dev test    # [[spec]...]                   run functional test (reserved for future use)
        \$ openpkg dev tools   # [path]                        tools to run (default to instance internals)
        \$ openpkg dev track   # [[spec]...]                   track version (formerly called vcheck)
        \$ openpkg dev update  #                               update development environment from master server
        \$ openpkg dev vim     # [[spec]...] [-f]              spec edit and lint (if HEAD or forced)
        --------------------------------------------------------------------------------------------------------
EOF
}

#   ------------------------------------------------------------

#   when invoked without any parameter, artifically inject "bash" command as default
if [ $# -eq 0 ]; then
    set -- bash
fi

#   extract command parameter
cmd="$1"
shift

#   create a unique id for unique temporary files
GUID=".`date +%Y%m%d%H%M%S`-$$"

#   make sure environment is set up
[ ".${OPENPKG_INST}" = . ] && die "OPENPKG_INST is not set";
[ ".${OPENPKG_WORK}" = . ] && die "OPENPKG_WORK is not set";
[ ".${OPENPKG_REPO}" = . ] && die "OPENPKG_REPO is not set";
[ ".${OPENPKG_DIST}" = . ] && die "OPENPKG_DIST is not set";

#   handle command
case ${cmd} in
    bash    | ba* ) cmd="bash"    ;;
    build   | bu* ) cmd="build"   ;;
    branch  | br* ) cmd="branch"  ;;
    clean   | cl* ) cmd="clean"   ;;
    diff    | di* ) cmd="diff"    ;;
    execute | ex* ) cmd="execute" ;;
    fetch   | fe* ) cmd="fetch"   ;;
    help    | he* ) cmd="help"    ;;
    install | in* ) cmd="install" ;;
    kill          ) cmd="kill"    ;;
    lint    | li* ) cmd="lint"    ;;
    list    | ls* ) cmd="list"    ;;
    new     | ne* ) cmd="new"     ;;
    package | pa* ) cmd="package" ;;
    peek    | pe* ) cmd="peek"    ;;
    release | re* ) cmd="release" ;;
    run     | ru* ) cmd="run"     ;;
    search  | se* ) cmd="search"  ;;
    setup         ) cmd="setup"   ;;
    test    | te* ) cmd="test"    ;;
    track   | tr* ) cmd="track"   ;;
    tools   | to* ) cmd="tools"   ;;
    update  | up* ) cmd="update"  ;;
    vcheck  | vc* ) cmd="vcheck"  ;;
    vim     | vi* ) cmd="vim"     ;;
    srcdir  | sr* ) cmd="srcdir"  ;;
    *) die "unknown command \"${cmd}\"" ;;
esac
${cmd} "$@"
exit $?

##
##  MANUAL PAGE
##

=pod

=head1 NAME

B<openpkg dev> - OpenPKG Development Shell

=head1 SYNOPSIS

B<openpkg dev> [I<dev-opt> ...] [I<cmd> [I<cmd-opt> ...] [I<cmd-arg> ...]]

=head1 DESCRIPTION

B<openpkg dev> is the Development Shell of the OpenPKG Tool Chain.
It is used by OpenPKG developers to work conviniently with the
OpenPKG packaging sources from the OpenPKG CVS Repository.

=head1 GLOBAL OPTIONS

The following global command options I<opt> exist:

=over 4

=item B<-v>, B<--verbose>

Enable verbose processing.

=item B<-q>, B<--quiet>

Enable quiet processing.

=item B<-h>, B<--help>

Display brief usage message.

=item B<-V>, B<--version>

Display command version.

=item B<--inst=>I<dir-list>

Specify white-space separated list of directories where OpenPKG
instances can be found. The current default is "C</openpkg-dev
/openpkg-20 /openpkg-13 /cw13 /cw12 /cw11 /cw10>". This can be
overridden in F<~/openpkg/dev.rc> with variable C<OPENPKG_INST>.

=item B<--work=>

FIXME
OPENPKG_WORK

=item B<--temp=>

FIXME
OPENPKG_TEMP

=item B<--mode=>

FIXME
OPENPKG_MODE=

=item B<--repo=>

FIXME
OPENPKG_REPO

=item B<--dist=>

FIXME
OPENPKG_DIST

=item B<--spec=>

FIXME
OPENPKG_SPEC

=item B<--ctag=>

FIXME
OPENPKG_CTAG

=item B<--exec=>

FIXME
OPENPKG_EXEC

=back

=head1 FILES

The following paths are used by B<openpkg dev>:

=over 4

=item F<~/.openpkg/dev.rc>

=item F<~/work/openpkg/>

=item F</openpkg-dev>

=back

=head1 SEE ALSO

bash(1), C<openpkg man rpm>.

=head1 AUTHOR

Thomas Lotterer E<lt>thl@openpkg.orgE<gt>

=cut

